@@ -43,7 +43,8 @@ impl AzblobWriter {
     }
 }
 
-#[async_trait]
+#[cfg_attr(not(target_arch = "wasm32"), async_trait)]
+#[cfg_attr(target_arch = "wasm32", async_trait(?Send))]
 impl oio::OneShotWrite for AzblobWriter {
     async fn write_once(&self, bs: &dyn oio::WriteBuf) -> Result<()> {
         let bs = oio::ChunkedBytes::from_vec(bs.vectored_bytes(bs.remaining()));
@@ -70,7 +71,8 @@ impl oio::OneShotWrite for AzblobWriter {
     }
 }
 
-#[async_trait]
+#[cfg_attr(not(target_arch = "wasm32"), async_trait)]
+#[cfg_attr(target_arch = "wasm32", async_trait(?Send))]
 impl oio::AppendObjectWrite for AzblobWriter {
     async fn offset(&self) -> Result<u64> {
         let resp = self
