@@ -193,7 +193,6 @@ impl kv::Adapter for Adapter {
             Capability {
                 read: true,
                 write: true,
-                create_dir: true,
 
                 ..Default::default()
             },
@@ -250,7 +249,7 @@ impl bb8::ManageConnection for MemcacheConnectionManager {
     async fn connect(&self) -> std::result::Result<Self::Connection, Self::Error> {
         let conn = TcpStream::connect(&self.address)
             .await
-            .map_err(parse_io_error)?;
+            .map_err(new_std_io_error)?;
         Ok(ascii::Connection::new(conn))
     }
 
@@ -262,7 +261,3 @@ impl bb8::ManageConnection for MemcacheConnectionManager {
         false
     }
 }
-
-pub fn parse_io_error(err: std::io::Error) -> Error {
-    Error::new(ErrorKind::Unexpected, &err.kind().to_string()).set_source(err)
-}
