@@ -19,5 +19,5 @@ mod backend;
 pub use backend::WebdavBuilder as Webdav;
 
 mod error;
-mod pager;
+mod lister;
 mod writer;
