@@ -40,6 +40,10 @@ pub enum Scheme {
     Azdls,
     /// [B2][crate::services::B2]: Backblaze B2 Services.
     B2,
+    /// [Seafile][crate::services::Seafile]: Seafile Services.
+    Seafile,
+    /// [Upyun][crate::services::Upyun]: Upyun Services.
+    Upyun,
     /// [cacache][crate::services::Cacache]: cacache backend support.
     Cacache,
     /// [cloudflare-kv][crate::services::CloudflareKv]: Cloudflare KV services.
@@ -68,6 +72,8 @@ pub enum Scheme {
     Hdfs,
     /// [http][crate::services::Http]: HTTP backend.
     Http,
+    /// [huggingface][crate::services::Huggingface]: Huggingface services.
+    Huggingface,
     /// [alluxio][created::services::Alluxio]: Alluxio services.
     Alluxio,
 
@@ -201,6 +207,8 @@ impl Scheme {
             Scheme::Hdfs,
             #[cfg(feature = "services-http")]
             Scheme::Http,
+            #[cfg(feature = "services-huggingface")]
+            Scheme::Huggingface,
             #[cfg(feature = "services-ipfs")]
             Scheme::Ipfs,
             #[cfg(feature = "services-ipmfs")]
@@ -235,6 +243,10 @@ impl Scheme {
             Scheme::Rocksdb,
             #[cfg(feature = "services-s3")]
             Scheme::S3,
+            #[cfg(feature = "services-seafile")]
+            Scheme::Seafile,
+            #[cfg(feature = "services-upyun")]
+            Scheme::Upyun,
             #[cfg(feature = "services-sftp")]
             Scheme::Sftp,
             #[cfg(feature = "services-sled")]
@@ -303,6 +315,7 @@ impl FromStr for Scheme {
             "gridfs" => Ok(Scheme::Gridfs),
             "hdfs" => Ok(Scheme::Hdfs),
             "http" | "https" => Ok(Scheme::Http),
+            "huggingface" | "hf" => Ok(Scheme::Huggingface),
             "ftp" | "ftps" => Ok(Scheme::Ftp),
             "ipfs" | "ipns" => Ok(Scheme::Ipfs),
             "ipmfs" => Ok(Scheme::Ipmfs),
@@ -321,6 +334,8 @@ impl FromStr for Scheme {
             "redis" => Ok(Scheme::Redis),
             "rocksdb" => Ok(Scheme::Rocksdb),
             "s3" => Ok(Scheme::S3),
+            "seafile" => Ok(Scheme::Seafile),
+            "upyun" => Ok(Scheme::Upyun),
             "sftp" => Ok(Scheme::Sftp),
             "sled" => Ok(Scheme::Sled),
             "supabase" => Ok(Scheme::Supabase),
@@ -357,6 +372,7 @@ impl From<Scheme> for &'static str {
             Scheme::Gridfs => "gridfs",
             Scheme::Hdfs => "hdfs",
             Scheme::Http => "http",
+            Scheme::Huggingface => "huggingface",
             Scheme::Foundationdb => "foundationdb",
             Scheme::Ftp => "ftp",
             Scheme::Ipfs => "ipfs",
@@ -376,6 +392,7 @@ impl From<Scheme> for &'static str {
             Scheme::Redis => "redis",
             Scheme::Rocksdb => "rocksdb",
             Scheme::S3 => "s3",
+            Scheme::Seafile => "seafile",
             Scheme::Sftp => "sftp",
             Scheme::Sled => "sled",
             Scheme::Supabase => "supabase",
@@ -390,6 +407,7 @@ impl From<Scheme> for &'static str {
             Scheme::Sqlite => "sqlite",
             Scheme::Mongodb => "mongodb",
             Scheme::Alluxio => "alluxio",
+            Scheme::Upyun => "upyun",
             Scheme::Custom(v) => v,
         }
     }
