@@ -28,7 +28,7 @@ fi
 # tar source code
 release_version=${OPENDAL_VERSION}
 # rc versions
-rc_version=${OPENDAL_VERSION_RC:rc.1}
+rc_version="${OPENDAL_VERSION_RC:-rc.1}"
 # Corresponding git repository branch
 git_branch=release-${release_version}-${rc_version}
 
@@ -38,8 +38,26 @@ mkdir -p dist/
 echo "> Checkout version branch"
 git checkout -B "${git_branch}"
 
+# Please update this part while package list changed.
+echo "> Update Cargo.toml to remove not released members"
+PACKAGES=(
+    "bindings\/c"
+    "bindings\/ruby"
+    "bindings\/haskell"
+    "bindings\/lua"
+    "bindings\/dotnet"
+    "bindings\/ocaml"
+    "bindings\/php"
+    "bindings\/cpp"
+    "bin\/"
+    "integrations\/"
+)
+for package in "${PACKAGES[@]}"; do
+  sed -i "/${package}/d" Cargo.toml
+done
+
 echo "> Start package"
-git archive --format=tar.gz --output="dist/apache-opendal-incubating-$release_version-src.tar.gz" --prefix="apache-opendal-incubating-$release_version-src/" "$git_branch"
+git archive --format=tar.gz --output="dist/apache-opendal-incubating-$release_version-src.tar.gz" --prefix="apache-opendal-incubating-$release_version-src/" --add-file=Cargo.toml "$git_branch"
 
 cd dist
 echo "> Generate signature"
